# -*- coding: utf-8 -*-
"""
    Module: E5A data analysis
    File:   e5_Gaf.py
    Descr.: E5A解析: Gaf-film analysys
    Update: 210710 build start Ay
            210725 v1.0
            210825 v1.1 chg oR -> oG で解析
            220116 v1.2 DB_dir changed
            230103 v1.3 refactored
    Last:   230223 v1.4 add Excel out

ref) 神原 Gaf報告_生物C,Ar,Fe_HDV2,EBT3_190417.pdf
      カタログ値  :  神原実測(C,Ar,Fe)
 HDV2:10-1000Gy: R:max 100Gy, ODovf 0.4 G:200, 0.5 B:50-300, 0.2
 EBT3: 0.1-10Gy: R:max   2Gy, ODovf 0.3 G: 10, 0.5 B:使えない

@author: Ayoshida(RIKEN,Nishina center)
"""

""" import: standard lib.       
----.---.---.---.---.---.---.---.---.---.---.---.---"""
import  pandas  as  pd  # used in MAIN helper funcs

""" import: my custom lib.       
----.---.---.---.---.---.---.---.---.---.---.---.---"""

""" import: from this package   
----.---.---.---.---.---.---.---.---.---.---.---.---"""
from gaf    import (anaOD,rdGafScan1,rdGafScan6,rdGafScan6_Kam)
from e5ana  import (e5_pptRep) 

""" .===.===.===.===.===.===.===.===.===.===.===.===.===.===.===.===
        class MyRep
====.===.===.===.===.===.===.===.===.===.===.===.===.===.===.=== === """
class MyRep( e5_pptRep ):
    """ Gaf-film report
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    
    def rep_scan(self, scn):
        """  rep. for scan images
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        """ pg.1 : im0[] """
        self.fnmF = 'org.scan image'
        self.newPage()
        im = scn.show_all(mode=0)
        self.wr_Image( im,(10,25),hgt=180)

        """ pg.2 : imCur[], imR,G,B[] """
        self.fnmF = 'Separated images & those RGB-band'
        self.newPage()
        im = scn.show_all(mode=2)
        self.wr_Image(im,(10,25),wid=250)
        scn.imRGB_all(mode=1)           # RGB colored image
        im = scn.show_all(mode=3)
        self.wr_Image(im,(10,70),wid=250)
        scn.imRGB_all(mode=0)           # RGG BW image に戻しておく

        """ pg.3 : imRm,Gm,Bm[] """
        self.fnmF = 'Masked images'
        self.newPage()
        im = scn.show_all(mode=4)
        self.wr_Image(im,(10,25),wid=250)

        """ pg.4 : imRod,God,Bod[] """
        self.fnmF = 'Opt.Dens.images'
        self.newPage()
        im = scn.show_all(mode=5)
        self.wr_Image(im,(10,25),wid=250)
        
    def rep_od(self, scn, rgb:str, imNo:int, WobON:bool, 
               odMin:float=None, odMax:float=None) -> object:
        """  rep. for O.D. image analysys
        scn:    rdGafScan6()    scan image
        rgb:    str     'R'/'G'/'B' band
        imNo:   int     0 .. scn.nim ; scan image number
        WobON:  bool    WobMag = ON:True / OFF:False
        odMin: float(0.0 ~ 1.0) if None, use self.odBG
        odMax: float(0.0 ~ 1.0) if None, use self.odMax
        <Ret.>
        aOD:    anaOD()     OD-ana. result for scn.od{R|G|B}[imNo]
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        if(rgb == 'R'): aOD = anaOD(scn.odR[imNo])
        if(rgb == 'G'): aOD = anaOD(scn.odG[imNo])
        if(rgb == 'B'): aOD = anaOD(scn.odB[imNo])

        """ # 以下 ban-xy をズラして確認したい場合
        aOD.set_band('x',(-20, 0,10,2))   # to check ban pos/wid 
        aOD.set_band('y',(-15,25,10,2))
        """
        """ pg.1 : plotXY """
        self.fnmF = 'od{}[{}]'.format(rgb,imNo)
        self.newPage()
        aOD.plotXY(odMin=odMin,odMax=odMax,plavr=True)            
        if(not WobON):
            self.wr_Figure(aOD.fig,(10,25), hgt=180 )
            """ pg.2 : plotFitGa """
            self.fnmF = 'od{}[{}]'.format(rgb,imNo)
            self.newPage()
            aOD._set_fig()                  # clear aOD.fig
            xcm,ycm = aOD.odCMm             # Center_of_Mass pos
            aOD.set_band('x',(ycm,ycm,1,3)) # set xy-band_start,_end,_step,_wid in mm 
            aOD.set_band('y',(xcm,xcm,1,3)) 
            aOD.plotXY(odMin=odMin,odMax=odMax,plavr=False)
            (xm,xsg,xa,xR2),(ym,ysg,ya,yR2) = aOD.plotFitGa()
            tx1 = 'GausFit ban-x,y: m,sg,a,R2='
            tx2 = '({:+.2f},{:.2f},{:.2f},{:.3f})'.format(xm,xsg,xa,xR2)
            tx3 = '({:+.2f},{:.2f},{:.2f},{:.3f})'.format(ym,ysg,ya,yR2)
            txt = tx1 +'\n'+ tx2 +'\n'+ tx3
            self.wr_Text(txt,(200,100))
        self.wr_Figure(aOD.fig,(10,25), hgt=180 )
        return( aOD )

    def save_scan(self,scn):
        """  save scan data 必要に応じて modes を変更して下さい
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        self.scn.save_all(modes=[2,11,12,14,15,31,32,41,42])

""" .===.===.===.===.===.===.===.===.===.===.===.===.===.===.===.===
         helper-func for MAIN : make report using MyRep()
====.===.===.===.===.===.===.===.===.===.===.===.===.===.===.=== === """
_fn_rep1 = '_repGaf1_scan.ppt'        # scan image report
_fn_rep2 = '_repGaf2_scanShift.ppt'   # scan Shift report
_fn_rep3 = '_repGaf3_anaOD.ppt'       # o.d. image report   

def mkReport(expID:str, runID:str, fn_scn:str, dpi:int, typ:int,
             sft:list, wob:list, rgb:str='G'):
    """  params below are given in e5DB_Gaf.py 
    expID:  str     experiment ID   like    '2301Kr'
    runID:  str     run ID          like    '2'
    fn_scn: str     scanned image file name '2301Kr_01_75dpi.bmp'
    dpi:    int     dot per inch            75
    typ:    int     typ =1:6枚組 =2:6枚(神原) =3:1枚
    sft:    list    crop shift for each image
    wob:    list    wobbler ON/OFF for each image
    rgb:    str     'R'/'G'/'B' band for analysis (defa.'G')
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""

    """ scan 6 image anal. """
    if(typ == 1):
        scn = rdGafScan6(fn_scn,dpi=dpi)
    if(typ == 2):
        scn = rdGafScan6_Kam(fn_scn,dpi=dpi)
    if(typ == 3):
        scn = rdGafScan1(fn_scn,dpi=dpi)
    fnmH = expID+runID
    fn_rep1 = '_' + fnmH + _fn_rep1
    fn_rep2 = '_' + fnmH + _fn_rep2
    fn_rep3 = '_' + fnmH + _fn_rep3
    myR = MyRep(expID,fnmH,hdr2=fn_rep1)
    myR.rep_scan(scn)
    myR.save_rep(fn=fn_rep1)

    """ scan x,y-shift & re-anal. """ 
    if(any(x is not None for x in sft)):
        for i,_sft in enumerate(sft):
            if(_sft is None): continue
            dx,dy=_sft
            # print(i,dx,dy)
            scn.scnShift(i,dx,dy)
        scn.scnAna_all()
        myR = MyRep(expID,fnmH,hdr2=fn_rep2)
        myR.rep_scan(scn)
        myR.save_rep(fn=fn_rep2)

    """ O.D. image anal """
    #myR = MyRep(scn,ttl,fn_rep3)
    myR = MyRep(expID,fnmH,hdr2=fn_rep3)
    aODL = []
    for i in range(scn.nim):
        #print('ODana=',i)
        if(wob[i] < 0): continue
        """　OD解析 param = rgb で解析 """
        aOD = myR.rep_od(scn,rgb,i, (wob[i] == 1) )    
        aODL.append( aOD )
    myR.save_rep(fn=fn_rep3)
    # myR.save_scan(scn)  .png, .csv を出力したい時用
    return(scn,aODL)     # return scn,aODL for later operations

""" .===.===.===.===.===.===.===.===.===.===.===.===.===.===.===.===
        helper func for MAIN : OD ana. result save to Excel
====.===.===.===.===.===.===.===.===.===.===.===.===.===.===.=== === """
_fn_xl  = 'anaOD.xlsx'      # anaOD report

def save_Excel(fnmH:str, aODL:list):
    """ scn.odG, ban-x,-y save to Excel
    fnmH:   str     file make header
    aODL:   [anaOD(),..]    ODana results of each image
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    with pd.ExcelWriter(fnmH+_fn_xl) as wr:
        for i,aOD in enumerate(aODL):
            snH = 'od({})_'.format(i)
            df1 = aOD.mkdf_Atrb()
            df2 = aOD.mkdf_odC()
            df1.to_excel(wr,sheet_name=snH+'Atrb',float_format="%.3f")
            df2.to_excel(wr,sheet_name=snH+'odC', float_format="%.3f")

""" ========================================================
            Main
    ========================================================"""
if __name__ == '__main__':
    from    e5ana   import(rDB_Gaf, get_RunDB_ix) 
    import  os
    """     HowTo:
    1) スキャンイメージ読取り結果 : _scan.ppt と　_scanShift.ppt の 
       pg.2 : 丸カーソルの位置を見比べて、Shift結果 が Gafフィルムの中心になるまで、
       e5DB_Gaf.py: rDB_Gaf[] の crop shift パラメータを調整する。
    2) 1)が完璧になれば、解析結果が _anaOD.ppt と _anaOD.xlsx に出力される。 
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    """ Run data DB
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    #DB_dir1 = 'Q:'          # mount ~/_pyE5A-ExpData/ as Q: drive
    DB_dir1 = './'         # Debug用 data DB dir
    DB_dir2 = '_GafData'    # Gaf data folder dir 
    """ Run DB 選択 ：　以下の方法から選択する ---.---.---.---"""
    #No = get_RunDB_ix(rDB_Gaf,'0000Test')  # for DeBug
    #No = get_RunDB_ix(rDB_Gaf,'2212Ar')    # ExpID 指定
    No = [0]                               # = [0] 最新の　Exp を解析
    #No = range(0,2)
    """-.---.---.---.---.---.---.---.---.---.---.---.---"""
    """ レポート作製 """
    """ O.D.解析に用いる色成分 'R'|'G'|'B' オススメ='G' """
    anaRGB = 'G'
    print('all No[] = ',No)
    for i in No:    
        expID,runID,[fn_scn,dpi,typ],sft,wob = rDB_Gaf[i]
        print('No[{}] expID={} runID={}'.format(i,expID,runID))
        fn = os.path.join(DB_dir1,expID)
        fn = os.path.join(fn,DB_dir2)
        fn = os.path.join(fn,fn_scn)
        scn,aODL = mkReport(expID,runID,fn,dpi,typ,sft,wob,anaRGB)
        """ scan result save """
        fnmH = '_'+expID+runID+'_'
        if(False):  # scanイメージ保存要なら = True
            # modes 指定は、gaf_scan.py:rdGafScan6.save_all()参照
            scn.save_all(fnmH,modes=[2,12,15,42])
        """ OD anal result save """
        if(True):  # OD anal Excel保存要なら = True
            save_Excel(fnmH, aODL)
    print('@@main: -- Fin ---')     # 以上です。
   
""" ========================================================
        ref) Numbered ref
    ========================================================
     
    ========================================================
        ref) Others
    ======================================================== 

------------------------------------------------------------------
"""