# -*- coding: utf-8 -*-
"""
    Module: E5A camac data analysis
    File:   e5Ppt_rep.py
    Descr.: e5Ana report using Python-pptx
    Update: 190619 v0.0 build for pyPTself
            191006 v0.1 mod for helper class
            210725 v1.0
    Last:   221228 v1.1 refactor page titles
@author: Ayoshida(RIKEN,Nishina center)
"""

""" import: standard lib.
----.---.---.---.---.---.---.---.---.---.---.---.---"""
        
""" import: python-pptx  package  (used v0.6.18)
    https://python-pptx.readthedocs.io/en/latest/
    > pip install python-pptx
----.---.---.---.---.---.---.---.---.---.---.---.---"""

""" import: my custom lib.       
----.---.---.---.---.---.---.---.---.---.---.---.---"""
from    aypylibSbs      import  (pptRep)

""" import from this module
----.---.---.---.---.---.---.---.---.---.---.---.---"""

""" .===.===.===.===.===.===.===.===.===.===.===.===.===.===.===.===
        Class   e5_PptRep       e5_*.py ppt report
====.===.===.===.===.===.===.===.===.===.===.===.===.===.===.=== === """
class e5_pptRep( pptRep ):
    """ Python-pptx helper class : simple ppt report
Params. for constructor:
    paper:  str     Paper size eg) 'A2','A3','A4'(defa))
Attributes:
    see ._myclear()
----.---.---.---.---.---.---.---.---.---.---.---.---.---"""

    """ class Attributes
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    def _clear_e5_pptRep(self): 
        self.fnPpt  = str()     # output .ppt file name
        self.expID  = str()     # ExpID
        self.fnmH   = str()     # Data file name header
        self.fnmF   = str()     # Data file name footer
        self.hdr1   = str()     # Header-1
        self.hdr2   = None      # Header-2
        self.ftr1   = None      # Footer-1
        self.ftr2   = None      # Footer-2
        self.pgN    = 0
        
        self._sld     = None    # current Slide()
        self._p_expID = list()  # ExpID   wrText params [(xpos,ypos),fs,fc]
        self._p_fnmH  = list()  # fname header
        self._p_fnmF  = list()  # fname footer
        self._p_hdr1  = list()  # Header-1
        self._p_hdr2  = list()  # Header-2
        self._p_ftr1  = list()  # Footer-1 
        self._p_ftr2  = list()  # Footer-2 
        self._p_pgN   = list()  # Page No
                 
    def __init__(self, expID:str, fnmH:str, fnmF:str=None,
                 hdr1:str=None, hdr2:str=None, ftr1:str=None, ftr2:str=None):
        """  Constructor
        expID:  str     Experiment ID       eg) '2210Kr'
        fnmH:   str     File name header    eg) 'scnEDssd01'
        fnmF:   str     File name fotter    eg) '-001: 3200.5 um'
        <optional>
        hdr1:   str     Header-1            eg) 'setup-1: 100-200, 300-2000um'
        hdr2:   str     Header-2            eg) 'setup-2: accumuration 120sec'
        ftr1:   str     Footer-1            eg) 'run comment-1: GOOD run'
        ftr2:   str     Footer-2            eg) 'run comment-2: Note to be checked'
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""        
        super().__init__('A4','land')
        self._clear_e5_pptRep()
        self.expID = expID
        self.fnmH  = fnmH
        self.fnmF  = fnmF
        self.hdr1  = hdr1
        self.hdr2  = hdr2
        self.ftr1  = ftr1
        self.ftr2  = ftr2
        self.set_Titles()
        self.fnPpt = '_' + expID + '_' + fnmH + '.ppt'
    
    def set_Titles(self,prm:list=None):
        """  set page header/footer attributes
        prm:    list    [[(left,top),fs,fc,it,bd,ul],.. len=8]
                        if None, set default values
            where,
            (left,top)  [mm] Left,Top corner pos
            fs: int     [Pt] font size, if None use self.fntSiz
            fc: str          font color eg) 'w','r', ...
            it: bool         Italic
            bd: bool         Bold
            ul: bool         Under Line
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        if(prm is None):
            #               (left,top), fs,  fc, it,    bd,    ul
            self._p_expID = [( 10,  5), 18, 'g', False, True,  False]
            self._p_fnmH  = [( 50,  5), 18, 'g', False, False, False]
            self._p_fnmF  = [(100,  5), 20, 'b', False, False, False]
            self._p_hdr1  = [( 50, 13), 16, 'k', True,  False, False]
            self._p_hdr2  = [(150, 13), 16, 'k', True,  False, False]
            self._p_ftr1  = [( 50,193), 16, 'k', True,  False, False] 
            self._p_ftr2  = [( 50,200), 16, 'k', True,  False, False]
            self._p_pgN   = [(250,200), 12, 'k', False, False, False]
        else:
            self._p_expID = prm[0]
            self._p_fnmH  = prm[1]
            self._p_fnmF  = prm[2]
            self._p_hdr1  = prm[3]
            self._p_hdr2  = prm[4]
            self._p_ftr1  = prm[5] 
            self._p_ftr2  = prm[6]
            self._p_pgN   = prm[7]

    def newPage(self):
        """  add new page, write page titles
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        self._sld = self.addSlide()
        self.pgN += 1
        self.wr_Titles()
       
    def wr_Titles(self):
        """  write page header/fotter
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        def _wr_ttl(txt,txp):
            if(txt is not None):
                self.wrText(self._sld, txt, 
                            lt=txp[0], fs=txp[1], fc=txp[2],
                            it=txp[3], bd=txp[4], ul=txp[5])
        
        _wr_ttl(self.expID, self._p_expID)
        _wr_ttl(self.fnmH,  self._p_fnmH)
        _wr_ttl(self.fnmF,  self._p_fnmF)
        _wr_ttl(self.hdr1,  self._p_hdr1)
        _wr_ttl(self.hdr2,  self._p_hdr2)
        _wr_ttl(self.ftr1,  self._p_ftr1)
        _wr_ttl(self.ftr2,  self._p_ftr2)
        _wr_ttl('- '+str(self.pgN)+' -',  self._p_pgN)

    def wr_Text(self, txt:str, lt:tuple, *,
               fs=None, fc=None, fn=None):
        """  write text box
        txt:    str          text to write
        lt:  (left,top) [mm] Left,Top corner pos
        <optional>
        fs:     int     [Pt] font size, if None use self.fntSiz
        fc:     str          font color eg) 'w','r', ...
        fn:     str          font name, if None use self.fntNm
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        self.wrText(self._sld, txt, lt, fs=fs, fc=fc, fn=fn )

    def wr_Image(self, im:object, lt:tuple, *,
                wid:int=None, hgt:int=None):
        """ write PIL Image
        im:     PIL.Image 
        lt:  (left,top) [mm] Left,Top corner pos
        <optional>
        wid:    int     [mm] width
        hgt:    int     [mm] height
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        self.wrImage(self._sld, im, lt, wid=wid, hgt=hgt)

    def wr_Figure(self, fig:object, lt:tuple, *,
                wid:int=None, hgt:int=None):
        """ write matplot.figure
        fig:    matplotlib.Figure 
        lt:  (left,top) [mm] Left,Top corner pos
        <optional>
        wid:    int     [mm] width
        hgt:    int     [mm] height
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        self.wrFigure(self._sld, fig, lt, wid=wid, hgt=hgt)
    
    def save_rep(self, fn:str=None):
        """ save report as .ppt
        fn:     str     save file name; defa) self.fnPpt
        ----.---.---.---.---.---.---.---.---.---.---.---.---"""
        fn = self.fnPpt if(fn is None) else fn
        self.save_ppt( fn )
        
        
""" ========================================================
            Test Main
    ========================================================"""
if __name__ == '__main__':

    """ Test-CTRL Sw ---.---.---.---.---.---.---.---.---"""
    SwTest01 =  True    # e5_pptRep()
    """-.---.---.---.---.---.---.---.---.---.---.---.---"""

    """ Run data DB
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    DB_dir1 = './'                  # test data DB dir
    ExpID   = '0000Test'
    DB_dir2 = '_RawDmp'             # dump file dir
    fnmH    = 'scnEDssd01'          # CAMAC dump file Head name

    """ test01 : e5_pptRep()
    ----.---.---.---.---.---.---.---.---.---.---.---.---"""
    if(SwTest01):
        fnmF = '-001: 3200.5 um'
        hdr1 = 'Header1: 100-200, 300-2000um'
        hdr2 = 'Header2: accumuration 120sec'
        ftr1 = 'Footer1: run comments 1'
        ftr2 = 'Footer2: run comments 2'
        rep = e5_pptRep(ExpID,fnmH,fnmF,hdr1,hdr2,ftr1,ftr2)
        rep.newPage()
        rep.wr_Text('sample text-1',(20,100),fs=20,fc='r')
        rep.wr_Text('sample text-2',(50,120),fs=30,fc='b',fn='ＭＳ Ｐ明朝')

    rep.save_rep('_a.pptx')
        
    print('@@main: -- Fin ---')     # 以上です。

""" ========================================================
        ref) Numbered ref
    ========================================================
r#01) Qiita Python 3の美しい関数引数システム
    https://qiita.com/raviqqe/items/ee2bcb6bef86502f8cc6
    
    ========================================================
        ref) Others
    ======================================================== 
------------------------------------------------------------------
"""
